IndexDirection

Enum IndexDirection 

pub enum IndexDirection {
    Derivative,
    Batch,
    Output,
    Input,
}
IndexDirection represents a tensor’s leg direction from the quantum circuit perspective.

In the context of quantum circuits and tensor networks, indices (or “legs”) of a tensor are categorized by their direction to signify their role in an operation.

While tensors may conceptually have multiple indices (legs), in the qudit-expr library, TensorExpression objects are always generated as 1-D, 2-D, or 3-D tensors. This enum links the conceptual/graphical tensor indices to the generated expression’s dimensions by grouping them along directions. So all Input, Output, and Batch legs correspond to a specific dimension of the generated tensor. This tensor network library can use this to predict and manipulate the shape of a generated tensor.

For example, kets capture quantum states, and are represented conceptually by column vectors. In the tensor network perspective, they are represented by a tensor with only Output indices. They have as many indices as there are qudits in the state and the size of each index is the radix of the qudit. For qubits, the radix is 2, so all indices would have dimension 2. However, when generated in the qudit-expr library, they are generated as a vector. The IndexDirection is what is used by the qudit-tensor library to expect a vector shape, since a ket tensor will only have indices in one direction.

Similarly, gates which capture quantum operations, are represented by unitary matrices. As a tensor, they have indices that point in both Input and Output directions. Even though there will likely be more than 2 indices, the qudit-tensor library expects a matrix to be generated by the underlying TensorExpression because of the two distinct directions.

Batch indices are used for measurements/Kraus operators or for batch executions of a network.

Variants



Derivative

For gradients and hessians



Batch

Batch index of a tensor. These are typically represented in quantum circuits through interactions with classical systems through measurements and/or feed-forward operations. Additionally, they can capture batch computations of a network.



Output

Output leg of a tensor, corresponds to rows of an operation. In circuit diagrams, these represent the wires leaving a gate or ket.



Input

Input leg of a tensor, corresponds to columns of an operation. In circuit diagrams, these represent the wires going into a gate or bra.

Implementations

Source

impl IndexDirection

pub fn is_derivative(&self) -> bool

pub fn is_batch(&self) -> bool

pub fn is_output(&self) -> bool

pub fn is_input(&self) -> bool