TensorIndex

Struct TensorIndex 

pub struct TensorIndex { /* private fields */ }
Represents an index of a tensor.

Indices in this format are associated with a direction, and as a result, an expected output or expected generation (input) shape and ordering. For example, a [super::network::QuditTensorNetwork] will store its output indices in this format because after evaluation, there will be an expected shape and ordering. The shape is determined by all the indices directions and sizes, and the ordering will be determined by the ids.

See Also

  • [super::tensor::QuditTensor] - Tensor object capturing how tensors are generated.
  • [super::network::QuditTensorNetwork] - Tensor network describing a desired calculation.

Implementations

Source

impl TensorIndex

pub fn new( direction: IndexDirection, index_id: IndexId, index_size: IndexSize, ) -> Self

Creates a new TensorIndex instance.

Arguments
  • direction - The direction of the tensor leg.
  • index_id - A unique identifier for the index.
  • index_size - The dimension or size of the index.
Returns

A new TensorIndex instance with the specified properties.

Examples
use qudit_expr::index::{IndexDirection, TensorIndex};

let idx = TensorIndex::new(IndexDirection::Input, 0, 2);
assert_eq!(idx.index_id(), 0);
assert_eq!(idx.index_size(), 2);
assert_eq!(idx.direction(), IndexDirection::Input);
pub fn direction(&self) -> IndexDirection

Returns the IndexDirection of the tensor leg.

Examples
use qudit_expr::index::{IndexDirection, TensorIndex};

let leg = TensorIndex::new(IndexDirection::Output, 1, 3);
assert_eq!(leg.direction(), IndexDirection::Output);
pub fn index_id(&self) -> IndexId

Returns the IndexId of the index associated with this leg.

Examples
use qudit_expr::index::{IndexDirection, TensorIndex};

let leg = TensorIndex::new(IndexDirection::Batch, 2, 4);
assert_eq!(leg.index_id(), 2);
pub fn index_size(&self) -> IndexSize

Returns the IndexSize of the index associated with this leg.

Examples
use qudit_expr::index::{IndexDirection, TensorIndex};

let leg = TensorIndex::new(IndexDirection::Input, 3, 5);
assert_eq!(leg.index_size(), 5);