IndexSize

Type Alias IndexSize 

pub type IndexSize = usize;
Represents the size or dimension of an index in a tensor network.

This type alias is used to specify the number of possible values for an index. From a quantum circuit perspective, Input and Output indices usually have size equal to radix of the qudit captured by the leg (2 for qubits, 3 for qutrits, etc); however, during contraction simplification, many edges can be grouped together to simplify the number of contractions, leading to a larger index size.