pub struct TensorExpression { /* private fields */ }Implementations
Sourceimpl TensorExpression
impl TensorExpression
pub fn new<T: AsRef<str>>(input: T) -> Self
pub fn from_raw(indices: Vec<TensorIndex>, inner: NamedExpression) -> Self
pub fn indices(&self) -> &[TensorIndex]
pub fn dimensions(&self) -> Vec<IndexSize> ⓘ
pub fn rank(&self) -> usize
pub fn generation_shape(&self) -> GenerationShape
Sourcepub fn tensor_strides(&self) -> Vec<usize>
pub fn tensor_strides(&self) -> Vec<usize>
Calculates the strides for each dimension of the tensor.
The stride for a dimension is the number of elements one must skip in the flattened array to move to the next element along that dimension. The strides are calculated such that the last dimension has a stride of 1, the second to last dimension has a stride equal to the size of the last dimension, and so on.
Examples
use qudit_expr::TensorExpression;
let mut tensor = TensorExpression::new("example() {[
[
[ 1, 0, 0, 0 ],
[ 0, 1, 0, 0 ],
[ 0, 0, 1, 0 ],
[ 0, 0, 0, 1 ],
], [
[ 1, 0, 0, 0 ],
[ 0, 1, 0, 0 ],
[ 0, 0, 0, 1 ],
[ 0, 0, 1, 0 ],
]
]}");
assert_eq!(tensor.tensor_strides(), vec![16, 8, 4, 2, 1]);pub fn reindex(&mut self, new_indices: Vec<TensorIndex>) -> &mut Self
pub fn permute( &mut self, perm: &[usize], redirection: Vec<IndexDirection>, ) -> &mut Self
pub fn stack_with_identity( &self, positions: &[usize], new_dim: usize, ) -> TensorExpression
pub fn partial_trace( &self, dimension_pairs: &[(usize, usize)], ) -> TensorExpression
Sourcepub fn substitute_parameters<S: AsRef<str>, C: AsRef<Expression>>(
&self,
variables: &[S],
values: &[C],
) -> Self
pub fn substitute_parameters<S: AsRef<str>, C: AsRef<Expression>>( &self, variables: &[S], values: &[C], ) -> Self
replace all variables with values, new variables given in variables input
pub fn destruct( self, ) -> (String, Vec<String>, Vec<ComplexExpression>, Vec<TensorIndex>)
Methods from Deref<Target = NamedExpression>