pub struct TensorIndex { /* private fields */ }Expand description
Represents an index of a tensor.
Indices in this format are associated with a direction, and as a result,
an expected output or expected generation (input) shape and ordering.
For example, a [super::network::QuditTensorNetwork] will store its output
indices in this format because after evaluation, there will
be an expected shape and ordering. The shape is determined by all the
indices directions and sizes, and the ordering will be determined by the ids.
See Also
[super::tensor::QuditTensor]- Tensor object capturing how tensors are generated.[super::network::QuditTensorNetwork]- Tensor network describing a desired calculation.
Implementations
Sourceimpl TensorIndex
impl TensorIndex
Sourcepub fn new(
direction: IndexDirection,
index_id: IndexId,
index_size: IndexSize,
) -> Self
pub fn new( direction: IndexDirection, index_id: IndexId, index_size: IndexSize, ) -> Self
Creates a new TensorIndex instance.
Arguments
direction- The direction of the tensor leg.index_id- A unique identifier for the index.index_size- The dimension or size of the index.
Returns
A new TensorIndex instance with the specified properties.
Examples
use qudit_expr::index::{IndexDirection, TensorIndex};
let idx = TensorIndex::new(IndexDirection::Input, 0, 2);
assert_eq!(idx.index_id(), 0);
assert_eq!(idx.index_size(), 2);
assert_eq!(idx.direction(), IndexDirection::Input);Sourcepub fn direction(&self) -> IndexDirection
pub fn direction(&self) -> IndexDirection
Returns the IndexDirection of the tensor leg.
Examples
use qudit_expr::index::{IndexDirection, TensorIndex};
let leg = TensorIndex::new(IndexDirection::Output, 1, 3);
assert_eq!(leg.direction(), IndexDirection::Output);Sourcepub fn index_id(&self) -> IndexId
pub fn index_id(&self) -> IndexId
Returns the IndexId of the index associated with this leg.
Examples
use qudit_expr::index::{IndexDirection, TensorIndex};
let leg = TensorIndex::new(IndexDirection::Batch, 2, 4);
assert_eq!(leg.index_id(), 2);Sourcepub fn index_size(&self) -> IndexSize
pub fn index_size(&self) -> IndexSize
Returns the IndexSize of the index associated with this leg.
Examples
use qudit_expr::index::{IndexDirection, TensorIndex};
let leg = TensorIndex::new(IndexDirection::Input, 3, 5);
assert_eq!(leg.index_size(), 5);Trait Implementations
Sourceimpl Clone for TensorIndex
impl Clone for TensorIndex
Sourcefn clone(&self) -> TensorIndex
fn clone(&self) -> TensorIndex
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for TensorIndex
impl Debug for TensorIndex
Sourceimpl Display for TensorIndex
impl Display for TensorIndex
Sourceimpl Hash for TensorIndex
impl Hash for TensorIndex
Sourceimpl Ord for TensorIndex
impl Ord for TensorIndex
Sourcefn cmp(&self, other: &TensorIndex) -> Ordering
fn cmp(&self, other: &TensorIndex) -> Ordering
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl PartialEq for TensorIndex
impl PartialEq for TensorIndex
Sourceimpl PartialOrd for TensorIndex
impl PartialOrd for TensorIndex
impl Copy for TensorIndex
impl Eq for TensorIndex
impl StructuralPartialEq for TensorIndex